import gmsh
import pathlib
import meshio
import tempfile


class GmshApi:

    def __init__(self):
        pass

    def __enter__(self):
        gmsh.initialize()

    def __exit__(self, exc_type, exc_val, exc_tb):
        gmsh.finalize()


def remesh_file(
    file_path: pathlib.Path, surface_loops: list[set[int]], mesh_size: float
):
    with GmshApi():
        # read mesh file
        gmsh.merge(str(file_path))

        # create topology and geometry
        gmsh.model.mesh.createTopology()
        gmsh.model.mesh.createGeometry()

        # create surface loops
        for surface_ids in surface_loops:
            surface_loop = gmsh.model.geo.addSurfaceLoop(list(surface_ids))
            gmsh.model.geo.addVolume([surface_loop])

        # synchronize added surface loops
        gmsh.model.geo.synchronize()

        field_id = gmsh.model.mesh.field.add("MathEval", 1)
        gmsh.model.mesh.field.setString(field_id, "F", str(mesh_size))
        gmsh.model.mesh.field.setAsBackgroundMesh(field_id)

        # generate 3D mesh
        gmsh.model.mesh.generate(3)

        with tempfile.TemporaryDirectory() as dir:
            file_path = pathlib.Path(dir) / "temp.msh"
            gmsh.write(str(file_path))

            return meshio.read(str(file_path))



from . import run_gmsh
import pathlib
from . import mesh
import lnmmeshio
import tempfile
import meshio
from .reorder import reorder


def combine_colored_stl_files(*stl_files: pathlib.Path) -> meshio.Mesh:
    """
    Combine multiple colored STL files into a single mesh that is returned.
    """
    return mesh.merge_colored_stl(*stl_files)[0]


def mesh_colored_stl_files(*stl_files: pathlib.Path, mesh_size: float) -> meshio.Mesh:
    """
    Generate a mesh from multiple colored STL files

    Parameters
    ----------
    *stl_files : pathlib.Path
        Paths to the STL files to be merged.

    mesh_size : float
        The target size for the mesh elements.

    Returns
    -------
    lnmmeshio.Discretization
        The generated mesh.
    """
    assert len(stl_files) > 0, "At least one STL file must be provided."

    with tempfile.TemporaryDirectory() as tmpdir:
        # merge stl files into a single mesh file
        gmsh_file = pathlib.Path(tmpdir) / "merged.mesh"
        m, surface_loops = mesh.merge_colored_stl(*stl_files)
        lnmmeshio.write_mesh(str(gmsh_file), m)

        # remesh file using Gmsh
        return run_gmsh.remesh_file(gmsh_file, surface_loops, mesh_size)
